#! /usr/bin/env node

var fs = require('fs');
var path = require('path');
var cgLib = require('./javascripts/lib');
var el = require('./javascripts/electron');
var ng = require('./javascripts/angular');
var js = require('./javascripts/jsApp');



var operation="", modulename="", itemName="",param1="",param2="",param3="",param4="";
var NL="\r\n\r\n", NT="\r\n\t", NTT="\r\n\t\t", NNT="\r\n\r\n\t", st="";

const basePath = process.cwd();
const version = "1.0.11";
var workingDir=""; 
var templatesPath = path.join(__dirname,"templates") , ngAppPath = path.join(basePath, "src");

ngAppPath = path.join(ngAppPath, "app");
workingDir = ngAppPath;


// Loading Arguments
var args= [];
var idx=0;
process.argv.forEach(function (val, index, array) {
    if(index>1) 
    {
        st = val.trim();
        if(st.length>0) 
        {
            args[idx] = st;
            idx++;
        }
    }
});


// Initiating parameters
args.forEach(function(p, index){
    if(index==0) operation = p;
    else if(index==1) modulename = p;
    else if(index==2) itemName = p;
    else if(index==3) param1 = p;
    else if(index==4) param2 = p;
    else if(index==5) param3 = p;
    else if(index==6) param4 = p;
});



// Changing parameters from long to small
if(operation=="generate") operation="ng";
if(operation=="electron") operation="el";
if(operation=="javascript") operation="js";

if(modulename=="component") modulename="c";
if(modulename=="service") modulename="s";
if(modulename=="layout") modulename="l";
if(modulename=="route") modulename="r";

if(modulename=="boilerplate") modulename="b";
if(modulename=="new") modulename="n";





// Navigating Commands

if(operation=="help" || operation=="h" || operation==""){
    help();
}
else if(operation.trim().length>0){

    // For Angular Application
    if( operation =="ng"){

        checkSrcAndAppDir();
        checkItemDirectory();

        ng.modulename = modulename; 
        ng.itemName = itemName; 
        ng.workingDir = workingDir; 
        ng.ngAppPath = ngAppPath;
        ng.templatesPath = templatesPath;
        
        
        if(modulename=="c"){
            ng.generateComponent();    
        }
        else if(modulename=="l"){
            ng.generateLayouts();    
        }
        else if(modulename=="r"){
            ng.generateRoutes();    
        }
        else if(modulename=="s"){
            ng.generateService();    
        }
    }

    // For Electron Application
    else if(operation=="el") {

        if(modulename=="b"){
            el.createElectronBoilerPlate(basePath,templatesPath, itemName, param1, param2);    
        }
    }
    // For Javascript Application
    else if(operation=="js") {

        if(modulename=="n"){
            js.createJavascriptApp(basePath,templatesPath, itemName, param1, param2);    
        }
    }
    else if(operation=="-v") {
        console.log("Version: " + version);
    }
    else if(operation=="dev"){
        getDevDetails();
    }

}
else help();





// HELP
function help(){

    var helpContent = cgLib.readFileSync(path.join(path.join(__dirname,"templates"),"help.html"));

    console.log("\r\n~~ CodeGenie Help ~~");
    console.log("~~ Version: "+ version +" ~~" + NL);

    var content = helpContent.split('//');
    content.forEach(function(val,index){
        if(val!=undefined && val!="undefined")
        { 
            var v = val.split('#')
            if(v[1]!= undefined)
            {
                if(v[0]!= undefined)console.log('\x1b[35m', v[0]);
                console.log('\x1b[36m', v[1]);
                if(v[2]!= undefined)console.log('\x1b[0m',"\t" + v[2]);
            }
        }
    });
    //console.log(helpContent);
}



function checkSrcAndAppDir(){

    // Checking src directory is exists
    var srcDir = path.join(basePath,"src");

    if (!fs.existsSync(srcDir)){
        fs.mkdirSync(srcDir);
    }

    // Checking app directory is exists
    if (!fs.existsSync(workingDir)){
        fs.mkdirSync(workingDir);
    }
}

function checkItemDirectory(){
    
    //console.log(itemName);

    if(itemName.indexOf('/')>-1){
        var itemPaths = itemName.split('/');

        for(var i=0;i<itemPaths.length-1;i++){

            var ipath = path.join(workingDir,itemPaths[i]);
            // console.log(ipath);
            if (!fs.existsSync(ipath)) fs.mkdirSync(ipath);            
            workingDir = ipath;
        }

        itemName = itemPaths[itemPaths.length-1];
        //console.log(itemName);
    }
}

function getDevDetails(){
    console.log("base path: " + basePath);
    console.log("ng path: " + ngAppPath);
    console.log("__dirname: " + __dirname);
    console.log("Templates path & isExists?: " + templatesPath + ", " + fs.existsSync(templatesPath).toString());
}

