const fs = require('fs');
const path = require('path');
const cgLib = require('./lib');

exports.itemName="";
exports.workingDir="";
exports.ngAppPath="";
exports.modulename="";
exports.templatesPath = "";


const NL="\r\n\r\n", NT="\r\n\t", NTT="\r\n\t\t", NNT="\r\n\r\n\t", st="";




// GENERATE COMPONENT
exports.generateComponent = function(){

    var items = exports.itemName.split(',');

    items.forEach(function(val, index){
        var newFileDir = path.join(exports.workingDir,val);
        if (!fs.existsSync(newFileDir)) fs.mkdirSync(newFileDir);
                
        var ts = getComponentTS(val);
        cgLib.writeFile(path.join(newFileDir,val + ".ts"), ts);
        cgLib.writeFile(path.join(newFileDir,val + ".html"), "<h2>" + val + "</h2>");
        
        updateAppModule(val, path.join(newFileDir,val));
    });
}

// GENERATE ROUTES
exports.generateRoutes = function(){
    
    var items = exports.itemName.split(',');


    // Checking layout folder
    var routeDir = path.join(exports.workingDir,"routes");
    if (!fs.existsSync(routeDir)) fs.mkdirSync(routeDir);

    var routes="", conf="", modName="";

    items.forEach(function(val, index){
                
        var routeTable = getRouteTS(val);
        cgLib.writeFile(path.join(routeDir,val + ".route.ts"), routeTable);        
        updateAppModule(val, path.join(routeDir,val + ".route"));

        modName = cgLib.getPascalCase(val);

        routes += "import { " +  modName + "Routes } from \"./routes/" + val +".route\";\r\n";
        conf += "\t{ path: '', component: "+ modName +"Layout, canActivate:[AuthGuard], data: { title: 'Secure Views' }, children: "+ modName +"Routes },\r\n";
    });


    // App Route TS
    var appRoutePath = path.join(exports.ngAppPath, "app.route.ts");

    if (!fs.existsSync(appRoutePath)){
        var appRoutes = "";
        appRoutes  += "import { ModuleWithProviders } from '@angular/core';\r\n";
        appRoutes  += "import { Routes, RouterModule } from '@angular/router';" + NL + NL;

        appRoutes += routes + NL

        appRoutes += "//Guard\r\n";
        appRoutes += "import { AuthGuard } from './authGuard.guard';" + NL;


        appRoutes += "const APP_ROUTE: Routes =[" + NT;
        appRoutes += "{ path: '', redirectTo: '/home', pathMatch: 'full', },\r\n";
        appRoutes += conf;
        appRoutes += "\t{ path: '**', redirectTo: 'notfound', pathMatch: 'full', }," + NL;
        appRoutes += "];" + NL;

        appRoutes += "export const Routing: ModuleWithProviders = RouterModule.forRoot(APP_ROUTE,{useHash: true});";

        cgLib.writeFile(appRoutePath, appRoutes);

    }

    // Auth Guard TS
    createAuthGuard();

    // Updating App Module
    var appModuleFile = path.join(exports.ngAppPath, "app.module.ts");
    if (fs.existsSync(appModuleFile)) {
        var data = cgLib.readFileSync(appModuleFile);

        var idx = data.indexOf("@NgModule({");
        if(idx>0) data = data.insertAt("//Routing\r\nimport { Routing } from './app.route';" + NL, idx-1);

        idx = data.indexOf("],",data.indexOf("imports:")+5);
        if(idx>0) data = data.insertAt(", Routing", idx);

        // Guard
        var guard = "//Guard\r\n";
        guard += "import { AuthGuard } from './authGuard.guard';" + NL;

        idx = data.indexOf("@NgModule({");
        if(idx>0) data = data.insertAt( guard, idx-1);

        cgLib.writeFileSync(appModuleFile,data);
    }
}

function createAuthGuard(){

    // Auth Guard
    var authGuardPath = path.join(exports.ngAppPath, "authGuard.guard.ts");
    if (!fs.existsSync(authGuardPath)){
        var auth = cgLib.readFileSync(path.join(exports.templatesPath,"authGuard.ts"));
        cgLib.writeFile(authGuardPath ,auth);        
    }

    // Config
    var configPath = path.join(exports.ngAppPath, "config.ts");
    if (!fs.existsSync(configPath)){
        var config = cgLib.readFileSync(path.join(exports.templatesPath,"config.ts"));
        cgLib.writeFile(configPath ,config);
    }
}


// GENERATE LAYOUTS
exports.generateLayouts = function(){
    
    var items = exports.itemName.split(',');

    // Checking layout folder
    var layoutDir = path.join(exports.workingDir,"layouts");
    if (!fs.existsSync(layoutDir)) fs.mkdirSync(layoutDir);
    

    items.forEach(function(val, index){
                
        var ts = getLayoutTS(val);
        var layoutHtml = getLayoutHtml();

        cgLib.writeFile(path.join(layoutDir,val + ".layout.ts"), ts);
        cgLib.writeFile(path.join(layoutDir,val + ".layout.html"), layoutHtml);
        
        updateAppModule(val, path.join(layoutDir,val + ".layout"));
    });
}



// GENERATE SERVICE
exports.generateService = function(){
    var newFileDir =  path.join(exports.ngAppPath, "services");
    
    if (!fs.existsSync(newFileDir)) fs.mkdirSync(newFileDir);    
    
    var items = exports.itemName.split(',');

    items.forEach(function(val, index){
        var ts = getServiceTS(val);
        cgLib.writeFile(path.join(newFileDir,val + ".service.ts"), ts);
        updateAppModule(val, path.join(newFileDir,val + ".service"));
    });
}



function getComponentTS(fileName){

    var code = "import { Component,OnInit } from '@angular/core';" + NL;
    var sel = fileName.toLowerCase();

    if(sel =="navbar" || sel =="sidebar" || sel =="header" || sel =="footer")
    {
        code +="@Component({\r\n\tselector:'"+ sel +"',\r\n\ttemplateUrl: '"+ fileName +".html'\r\n})";
    }
    else code +="@Component({\r\n\ttemplateUrl: '"+ fileName +".html'\r\n})";

    

    code += "\r\nexport class " + cgLib.getPascalCase(fileName) + " implements OnInit {" + NL;

    code += "\tconstructor() { }" +NL;
    code += "\tngOnInit() {" + NT;
    code += "}";

    code += NL + "}";
    return code;
}


function getRouteTS(fileName){
    var code = ""; 
    
    code += "import { ModuleWithProviders } from '@angular/core';\r\n";
    code += "import { Routes, RouterModule } from '@angular/router';" + NL + NL;

    code +="export const "+ cgLib.getPascalCase(fileName) +"Routes : Routes =[" + NL;
    code += "\t//{path: '',   component: }," + NL;
    code += "];";

    return code;
}

function getLayoutTS(fileName){
    var code = "import { Component,OnInit } from '@angular/core';" + NL;
    code +="@Component({\r\n\ttemplateUrl: '"+ fileName +".layout.html'\r\n})";

    code += "\r\nexport class " + cgLib.getPascalCase(fileName) + "Layout implements OnInit {" + NL;

    code += "\tconstructor() { }" +NL;
    code += "\tngOnInit() {" + NT;
    code += "}";

    code += NL + "}";
    return code;
}


function getServiceTS (serviceName){
    var code = "import { Injectable } from '@angular/core';\r\n";
    code += "import { Http, Headers, Response } from '@angular/http';\r\n";
    code += "import { Observable } from 'rxjs/Rx';\r\n";
    code += "import 'rxjs/add/operator/map';" + NL + NL;

    code +="@Injectable()\r\n";
    code += "export class " + cgLib.getPascalCase(serviceName) + "Service{" + NL;

    code += "\tapiUrl: string = \"http://localhost/api/\";" + NL;
    code += "\tconstructor(private http: Http) { }" +NL;


    code += NL + "\t//Save " + serviceName + NT;
    code += "save"+ serviceName +"(){" + NTT;
    code += "return this.http.post(this.apiUrl + \"\",{" +NTT;
    code += "}," + NTT;
    code += "{headers: new Headers({'X-Requested-With': 'XMLHttpRequest', 'Content-Type': 'application/json'})}" + NTT;
    code += ");" + NT + "}";

    code += NL + NL + "}";
    return code;
}


function getLayoutHtml (){
    var html = "<!-- Navbar -->\r\n";
    html += "<navbar></navbar>" + NL;

    html += "<div class=\"row\">" + NL;
    html +="<!-- Sidebar -->\r\n";
    html += "<div class=\"col-sm-4 col-md-4 col-lg-2\">" + NT;
    html += "<sidebar></sidebar>\r\n";
    html += "</div>" + NL;

    html +="<!-- Content -->\r\n";
    html += "<div class=\"col-sm-8 col-md-8 col-lg-10\">" + NT;
    html += "<router-outlet></router-outlet>\r\n";
    html += "</div>" + NL;
    html += "</div>";
    
    return html;
}

function updateAppModule(className, filePath){

    className = cgLib.getPascalCase(className);

    

    filePath =  "./" + filePath.substr(exports.ngAppPath.length+1).replace(/\\/g, "/");

    var code = "";
    var idx=0;


    var appModuleFile = path.join(exports.ngAppPath, "app.module.ts");
    if (!fs.existsSync(appModuleFile))
    {
        console.log("app module not found");
        return;
    }    


    var data = cgLib.readFileSync(appModuleFile);


    if(exports.modulename=="c") {
        if(data.indexOf("// Components")==-1) {
            idx = data.indexOf("@NgModule({");
            data = data.insertAt( NL + "// Components\r\n ",idx);        
        }

        code = "import { " +  className + " } from \"" + filePath + "\";\r\n";
    }
    else if(exports.modulename=="l") {
        if(data.indexOf("// Layouts")==-1) {
            idx = data.indexOf("@NgModule({");
            data = data.insertAt( NL + "// Layouts\r\n ",idx);        
        }

        code = "import { " +  className + "Layout } from \"" + filePath + "\";\r\n";
    }   
    else if(exports.modulename=="s"){ 

        if(data.indexOf("// Services")==-1) {
            idx = data.indexOf("@NgModule({");
            data = data.insertAt( NL + "// Services\r\n ",idx);        
        }

        code = "import { " +  className + "Service } from \"" + filePath + "\";\r\n";
    }
    

    // Skip if exists
    if(data.indexOf(filePath)>-1) return;



    if(exports.modulename=="c"){

        idx = data.indexOf("// Components");
        if(idx>0) data = data.insertAt(code, idx+14);    

        idx = data.indexOf("],",data.indexOf("declarations: [")+5);
        if(idx>0) data = data.insertAt( ", " + className, idx);
    }
    else if(exports.modulename=="l"){
        
        idx = data.indexOf("// Layouts");
        if(idx>0) data = data.insertAt(code, idx+12);    

        idx = data.indexOf("],",data.indexOf("declarations: [")+5);
        if(idx>0) data = data.insertAt( ", " + className + "Layout", idx);
    }
    else if(exports.modulename=="s"){

        idx = data.indexOf("// Services");
        if(idx>0) data = data.insertAt(code, idx+14);    


        className += "Service";
        idx = data.indexOf("],",data.indexOf("providers:")+3);
        if(idx>0) data = data.insertAt( ", " + className , idx);
    }


    data = data.insertAt("\r\n", data.indexOf("@NgModule(")-1);
    // clean up
    data = data.replace(NL + NL,NL);
    data = data.replace(NL + "\r\n",NL);
    
    cgLib.writeFileSync(appModuleFile, data);
}