var fs = require('fs');
var path = require('path');
var cgLib = require('./lib');

exports.createElectronBoilerPlate = function(filePath,templatesPath,appName,authorName,email){


    // loading template files content
    var html = cgLib.readFileSync(path.join(templatesPath,"el.html"));
    var mainJs = cgLib.readFileSync(path.join(templatesPath,"el.js"));
    var package = cgLib.readFileSync(path.join(templatesPath,"el-package.json"));
    var license = cgLib.readFileSync(path.join(templatesPath,"license"));
    var readme = cgLib.readFileSync(path.join(templatesPath,"readme.md"));


    // Checking folder exists
    var appDir = path.join(filePath,"app");
    if (!fs.existsSync(appDir)) fs.mkdirSync(appDir);

    var assetDir = path.join(appDir,"assets");
    if (!fs.existsSync(assetDir)) fs.mkdirSync(assetDir);

    var templateDir = path.join(appDir,"templates");
    if (!fs.existsSync(templateDir)) fs.mkdirSync(templateDir);

    var javascriptDir = path.join(appDir,"javascripts");
    if (!fs.existsSync(javascriptDir)) fs.mkdirSync(javascriptDir);

    var cssDir = path.join(assetDir,"css");
    if (!fs.existsSync(cssDir)) fs.mkdirSync(cssDir);

    var iconsDir = path.join(assetDir,"icons");
    if (!fs.existsSync(iconsDir)) fs.mkdirSync(iconsDir);


    // Replacing appname, email if given
    if(appName.length>0)
    {
        appName = appName.toLowerCase();
        package = package.replace("\"name\": \"app\",", "\"name\": \"" + appName + "\",");
        package = package.replace("\"name\": \"\",", "\"name\": \"" + authorName + "\",");
        package = package.replace("\"email\": \"\",", "\"email\": \"" + email + "\",");


        license = license.replace("<%= name %>",authorName);
        license = license.replace("<%= email %>",email);

        readme = readme.replace("My superb app", appName);
        readme = readme.replace("<%= name %>", authorName);
    }


    // Writting fies
    cgLib.writeFile(path.join(filePath, "package.json") ,package);
    cgLib.writeFile(path.join(filePath, "license") ,license);
    cgLib.writeFile(path.join(filePath, "readme.md") ,readme);
    cgLib.writeFile(path.join(filePath, ".gitignore") ,"node_modules/dist");
    cgLib.writeFile(path.join(javascriptDir, "mainWindow.js") ,"const { BrowserWindow } = require('electron');");
    cgLib.writeFile(path.join(cssDir, "app.css"), "");
    

    cgLib.writeFile(path.join(appDir, "main.js") ,mainJs);
    cgLib.writeFile(path.join(templateDir, "index.html") ,html);

}


